{****************************************************************************
                           "Stoichiometry Calculator"

 Program Description

 This program is a calculator for stoichiometry problems that all chemistry
 teachers and students face. Calculations that might once take ten minutes
 now only takes seconds with the calculator. When this calculator is in use
 in the school, it will save all chemistry teachers and students hours of
 time that are wasted on manual calculation.
 ****************************************************************************}

program stoicalc;

{$X+} {Turn on the extended syntax}
{$V-} {Turn off strict string length checking}

{Uses the following units in the program}
uses
  app,       {Turbo Vision unit}
  calc,      {Calculator unit}
  dedialog,  {Data entry dialog unit}
  dialogs,   {Turbo Vision unit}
  drivers,   {Turbo Vision unit}
  elemdata,  {Element data unit}
  equation,  {Unit and equation unit}
  menus,     {Turbo Vision unit}
  msgbox,    {Turbo Vision unit}
  objects,   {Turbo Vision unit}
  register,
  stddef,    {Standard definition unit}
  stdlib,    {Standard library unit}
  stdolib,
  stoicdef,  {Stoichiometry calculator definition unit}
  stoiclib,
  unitdata,  {Unit data (eg. amount, mass, volume) unit}
  views;     {Turbo Vision unit}

type

  TCalculateDataRec = record
    UnitListBox : TListboxRec;  {Listbox}
    FindListBox : TListboxRec;  {Listbox}
    SF : Word;                  {Checkbox}
  end;

  {Red button}
  PTBlueButton = ^TBlueButton;
  TBlueButton = object(TButton)
    function GetPalette: PPalette; virtual;
  end;

  {The project window object}
  PTProjectWindow = ^TProjectWindow;
  TProjectWindow = object(TWindow)
    CalculateDataRec : TCalculateDataRec;
    FindTopItem, UnitTopItem : Integer;
    PEquation : PTEquation;

    constructor Init(VPEquation : PTEquation);
    destructor Done; virtual;
    procedure Calculate;
    procedure ClearAllData;
    procedure DataEntry(Which: byte; IsReactant:boolean);
    function GetPalette: PPalette; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    function RealToStr(i : real) : string;
    procedure SetYield;
    function Valid(Command: Word): Boolean; virtual;
  end;

  {The application object}
  TApp = object(TApplication)
    constructor Init;
    procedure About;
    procedure EndModal(Command: Word); virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure InitMenuBar; virtual;
    procedure InitStatusLine; virtual;
    procedure NewWindow;
  end;

{----------------------------------------------------------------------------}

var
  VApp : TApp;

{----------------------------------------------------------------------------}

  function GetApplication : PApplication;
  begin
    GetApplication := @VApp;
  end;

{----------------------------------------------------------------------------}

function TBlueButton.GetPalette : PPalette;
const
  CBlueButton = #30#11#19#13#14#14#14#15;
  P : string[Length(CBlueButton)] = CBlueButton;
begin
  GetPalette := @P;
end;
                    { cbutton cdialog tbutton }
{----------------------------------------------------------------------------}

{This is the constructor to the project window object. It constructs the
 project window visuals but does not display it on the screen yet}

constructor TProjectWindow.Init(VPEquation : PTEquation);
var R : TRect;
    Control, Labl, Histry : PView;
begin

  R.Assign(4,1,74,21);
  inc(numWindowsOpened);
  TWindow.Init(R, 'Main Window', numWindowsOpened);
  Options := Options or ofCentered;
  Flags := Flags and not (wfGrow or wfZoom);
  setState(sfShadow, false);
  hideCursor;

  with CalculateDataRec do begin
    UnitListBox.Focused := 0;
    FindListBox.Focused := 0;
    SF := Word(TRUE);
  end;
  FindTopItem := 0;
  UnitTopItem := 0;

  PEquation := VPEquation;
  with PEquation^ do begin

    R.Assign(4,2,63,3);
    Control := New(PStaticText, Init(R, Name));
    Insert(Control);

    R.Assign(4,4,67,5);
    Control := New(PStaticText, Init(R,
      'Press one of the buttons (with the Space Bar) to enter data:'));
    Insert(Control);

    if (numReactants <= 5) and (numProducts <= 5) then begin

      R.Assign(3,6,35,8);
      Control := New(PButton, Init(R, reactant[1].name, cmR1Button,
        bfDefault));
      Insert(Control);

      if numReactants >= 2 then begin
        R.Assign(3,8,35,10);
        Control := New(PButton, Init(R, reactant[2].name, cmR2Button, bfNormal));
        Insert(Control);
      end;

      if numReactants >= 3 then begin
        R.Assign(3,10,35,12);
        Control := New(PButton, Init(R, reactant[3].name, cmR3Button, bfNormal));
        Insert(Control);
      end;

      if numReactants >= 4 then begin
        R.Assign(3,12,35,14);
        Control := New(PButton, Init(R, reactant[4].name, cmR4Button, bfNormal));
        Insert(Control);
      end;

      if numReactants >= 5 then begin
        R.Assign(3,14,35,16);
        Control := New(PButton, Init(R, reactant[5].name, cmR5Button, bfNormal));
        Insert(Control);
      end;

      R.Assign(35,6,67,8);
      Control := New(PButton, Init(R, product[1].name, cmP1Button, bfNormal));
      Insert(Control);

      if numProducts >= 2 then begin
        R.Assign(35,8,67,10);
        Control := New(PButton, Init(R, product[2].name, cmP2Button, bfNormal));
        Insert(Control);
      end;

      if numProducts >= 3 then begin
        R.Assign(35,10,67,12);
        Control := New(PButton, Init(R, product[3].name, cmP3Button, bfNormal));
        Insert(Control);
      end;

      if numProducts >= 4 then begin
        R.Assign(35,12,67,14);
        Control := New(PButton, Init(R, product[4].name, cmP4Button, bfNormal));
        Insert(Control);
      end;

      if numProducts >= 5 then begin
        R.Assign(35,14,67,16);
        Control := New(PButton, Init(R, product[5].name, cmP5Button, bfNormal));
        Insert(Control);
      end;

    end else begin
      R.Assign(3,6,35,8);
      Control := New(PButton, Init(R, reactant[1].name, cmR1Button,
        bfDefault));
      Insert(Control);

      if numReactants >= 2 then begin
        R.Assign(3,7,35,9);
        Control := New(PTBlueButton, Init(R, reactant[2].name, cmR2Button, bfNormal));
        Insert(Control);
      end;

      if numReactants >= 3 then begin
        R.Assign(3,8,35,10);
        Control := New(PButton, Init(R, reactant[3].name, cmR3Button, bfNormal));
        Insert(Control);
      end;

      if numReactants >= 4 then begin
        R.Assign(3,9,35,11);
        Control := New(PTBlueButton, Init(R, reactant[4].name, cmR4Button, bfNormal));
        Insert(Control);
      end;

      if numReactants >= 5 then begin
        R.Assign(3,10,35,12);
        Control := New(PButton, Init(R, reactant[5].name, cmR5Button, bfNormal));
        Insert(Control);
      end;

      if numReactants >= 6 then begin
        R.Assign(3,11,35,13);
        Control := New(PTBlueButton, Init(R, reactant[6].name, cmR6Button, bfNormal));
        Insert(Control);
      end;

      if numReactants >= 7 then begin
        R.Assign(3,12,35,14);
        Control := New(PButton, Init(R, reactant[7].name, cmR7Button, bfNormal));
        Insert(Control);
      end;

      if numReactants >= 8 then begin
        R.Assign(3,13,35,15);
        Control := New(PTBlueButton, Init(R, reactant[8].name, cmR8Button, bfNormal));
        Insert(Control);
      end;

      if numReactants = 9 then begin
        R.Assign(3,14,35,16);
        Control := New(PButton, Init(R, reactant[9].name, cmR9Button, bfNormal));
        Insert(Control);
      end;

      R.Assign(35,6,67,8);
      Control := New(PButton, Init(R, product[1].name, cmP1Button, bfNormal));
      Insert(Control);

      if numProducts >= 2 then begin
        R.Assign(35,7,67,9);
        Control := New(PTBlueButton, Init(R, product[2].name, cmP2Button, bfNormal));
        Insert(Control);
      end;

      if numProducts >= 3 then begin
        R.Assign(35,8,67,10);
        Control := New(PButton, Init(R, product[3].name, cmP3Button, bfNormal));
        Insert(Control);
      end;

      if numProducts >= 4 then begin
        R.Assign(35,9,67,11);
        Control := New(PTBlueButton, Init(R, product[4].name, cmP4Button, bfNormal));
        Insert(Control);
      end;

      if numProducts >= 5 then begin
        R.Assign(35,10,67,12);
        Control := New(PButton, Init(R, product[5].name, cmP5Button, bfNormal));
        Insert(Control);
      end;

      if numProducts >= 6 then begin
        R.Assign(35,11,67,13);
        Control := New(PTBlueButton, Init(R, product[6].name, cmP6Button, bfNormal));
        Insert(Control);
      end;

        if numProducts >= 7 then begin
          R.Assign(35,12,67,14);
          Control := New(PButton, Init(R, product[7].name, cmP7Button, bfNormal));
          Insert(Control);
        end;

        if numProducts >= 8 then begin
          R.Assign(35,13,67,15);
          Control := New(PTBlueButton, Init(R, product[8].name, cmP8Button, bfNormal));
          Insert(Control);
        end;

        if numProducts = 9 then begin
          R.Assign(35,14,67,16);
          Control := New(PButton, Init(R, product[9].name, cmP9Button, bfNormal));
          Insert(Control);
        end;

    end;

    R.Assign(3,17,19,19);
    Control := New(PButton, Init(R, 'Set ~Y~ield', cmSetYield, bfNormal));
    Insert(Control);

    R.Assign(19,17,35,19);
    Control := New(PButton, Init(R, 'C~a~lculate', cmCalculate, bfNormal));
    Insert(Control);

    R.Assign(35,17,53,19);
    Control := New(PButton, Init(R, 'C~l~ear All Data', cmClearAllData, bfNormal));
    Insert(Control);

    R.Assign(53,17,67,19);
    Control := New(PButton, Init(R, 'Cl~o~se', cmClose, bfNormal));
    Insert(Control);

    SelectNext(False);
  end; {with PEquation^ do begin}
end;

{----------------------------------------------------------------------------}

{This is the destructor to the project window object}

destructor TProjectWindow.Done;
begin
  TWindow.Done;
  dispose(PEquation, Done);
  dec(numWindowsOpened);
end;

{----------------------------------------------------------------------------}

{This procedure is called when the user presses the "Calculate" button}

procedure TProjectWindow.Calculate;
var

  Dlg             : PDialog;
  PFindCollection : PTUnsortedStringCollection;
  PFindTopItem    : PInteger;
  PUnit           : PTUnit;
  PUnitCollection : PTUnsortedStringCollection;
  PUnitTopItem    : PInteger;
  Result          : word;

  procedure NotEnoughDataMessageBox;
  begin
    MessageBox('You haven''t entered enough data. Calculations cannot '+
      'be made.', nil, mfError + mfOKButton);
  end;

  procedure ReportMessageBox(Number : real; _Unit : string);
  begin
    if Number = UNSET then
      NotEnoughDataMessageBox
    else
      MessageBox('There are ' + RealToStr(Number) + ' ' + _Unit + ' of ' +
        PUnit^.Name + '.', nil, mfInformation + mfOkButton);
  end;

  {This is the alternate form of the report messagebox procedure.
   Notice that the parameters and the text are arranged differently.}
  procedure ReportMessageBox_Alt(TypeName : string; Number : real;
    _Unit : string);
  begin
    if Number = UNSET then
      NotEnoughDataMessageBox
    else
      MessageBox('The ' + TypeName + ' of ' + PUnit^.Name + ' is ' +
        RealToStr(Number) + ' ' + _Unit + '.', nil, mfInformation +
        mfOkButton);
  end;

  function MakeUnitCollection : PTUnsortedStringCollection;
  var
    c : PTUnsortedStringCollection;
    n : byte;
  begin
    new(c, Init(1,1));
    for n := 1 to PEquation^.numReactants do
      c^.Insert(NewStr(PEquation^.Reactant[n].Name));
    for n := 1 to PEquation^.numProducts do
      c^.Insert(NewStr(PEquation^.Product[n].Name));
    MakeUnitCollection := c;
  end;

  function MakeFindCollection : PTUnsortedStringCollection;
  var c : PTUnsortedStringCollection;
  begin
    new(c, Init(25,0));
    with c^ do begin
      Insert(NewStr('Amount (mol)'));
      Insert(NewStr('Amount (atoms)'));
      Insert(NewStr('Amount (molecules)'));
      Insert(NewStr('Mass (mg)'));
      Insert(NewStr('Mass (g)'));
      Insert(NewStr('Mass (kg)'));
      Insert(NewStr('Volume (mL)'));
      Insert(NewStr('Volume (L)'));
      Insert(NewStr('Volume (kL)'));
      Insert(NewStr('Molarity (mol/mL)'));
      Insert(NewStr('Molarity (mol/L)'));
      Insert(NewStr('Molarity (mol/kL)'));
      Insert(NewStr('Density (g/mL)'));
      Insert(NewStr('Density (g/L)'));
      Insert(NewStr('Density (g/kL)'));
      Insert(NewStr('Density (kg/L)'));
      Insert(NewStr('Temperature (Kelvin)'));
      Insert(NewStr('Temperature (C)'));
      Insert(NewStr('Temperature (F)'));
      Insert(NewStr('Pressure (kPa)'));
      Insert(NewStr('Pressure (Pa)'));
      Insert(NewStr('Pressure (atm)'));
      Insert(NewStr('Pressure (torr)'));
      Insert(NewStr('Yield'));
      Insert(NewStr('Limiting Reagent'));
    end;
  MakeFindCollection := c;
end;


Begin {TProjectWindow.Calculate}

  Dlg := PDialog(RezFile.Get('CalculateDialog'));

  PUnitTopItem := @(PListBox(Dlg^.Last^.Prev^.Prev^.Prev)^.TopItem);
  PFindTopItem := @(PListBox(Dlg^.Last^.Prev^.Prev^.Prev^.Prev^.Prev^.Prev)^.
    TopItem);

  PUnitTopItem^ := UnitTopItem;
  PFindTopItem^ := FindTopItem;

  PUnitCollection := MakeUnitCollection;
  PFindCollection := MakeFindCollection;

  with CalculateDataRec do begin
    UnitListBox.List := PUnitCollection;
    FindListBox.List := PFindCollection;
  end;

  Result := GetApplication^.ExecuteDialog(Dlg, @CalculateDatarec);

  dispose(PUnitCollection, done);
  dispose(PFindCollection, done);

  UnitTopItem := PUnitTopItem^;
  FindTopItem := PFindTopItem^;

  if Result = cmCancel then
    exit;

  with CalculateDataRec do begin

    {If the user opts to find the yield, then}
    if FindListBox.Focused = 23 then begin
      PEquation^.FindYield;
      MessageBox('The Yield of the equation is ' +
        RealToStr(PEquation^.Yield), nil, mfInformation + mfOkButton);

    {If the user opts to find the limiting reagent, then}
    end else if FindListBox.Focused = 24 then begin
      PEquation^.FindLimitings;
      if PEquation^.PLimitingReactant = nil then
        NotEnoughDataMessageBox
      else
        MessageBox('The Limiting Reagent of the equation is ' +
          PEquation^.PLimitingReactant^.Name, nil, mfInformation +
          mfOkButton);

    {If the user chooses other options, then}
    end else begin

      {Using the information in the unit selection list box, set PUnit to
       point to the TUnit object that the user selects}

      if UnitListBox.Focused + 1 <= PEquation^.numReactants then
        PUnit := @PEquation^.Reactant[UnitListBox.Focused + 1]
      else
        PUnit := @PEquation^.Product[UnitListBox.Focused - PEquation^.numReactants + 1];

      {Calculate PUnit using stoichiometry. If not enough data has been
       entered, then display the "not enough data message box" and exit
       the procedure}

      if not PEquation^.SetAmountByStoichiometry(PUnit) then begin
        NotEnoughDataMessageBox;
        exit;
      end;

      {Depending on what the user selects to see, the following case
       structure displays the necessary information}
      case FindListBox.Focused of
        0 : ReportMessageBox(PUnit^.Amount.GetData(AMOUNT_MOL), 'mol');
        1 : ReportMessageBox(PUnit^.Amount.GetData(AMOUNT_ATOMS), 'atoms');
        2 : ReportMessageBox(PUnit^.Amount.GetData(AMOUNT_MOLECULES),
              'molecules');
        3 : ReportMessageBox(PUnit^.Mass.GetData(MASS_MG), 'mg');
        4 : ReportMessageBox(PUnit^.Mass.GetData(MASS_G), 'g');
        5 : ReportMessageBox(PUnit^.Mass.GetData(MASS_KG), 'kg');
        6 : ReportMessageBox(PUnit^.Volume.GetData(VOLUME_ML), 'mL');
        7 : ReportMessageBox(PUnit^.Volume.GetData(VOLUME_L), 'L');
        8 : ReportMessageBox(PUnit^.Volume.GetData(VOLUME_KL), 'kL');
        9 : ReportMessageBox_Alt('Molarity',
              PUnit^.Molarity.GetData(MOLARITY_MOLPML), 'mol/mL');
        10 : ReportMessageBox_Alt('Molarity',
               PUnit^.Molarity.GetData(MOLARITY_MOLPL), 'mol/L');
        11 : ReportMessageBox_Alt('Molarity',
               PUnit^.Molarity.GetData(MOLARITY_MOLPKL), 'mol/kL');
        12 : ReportMessageBox_Alt('Density',
               PUnit^.Density.GetData(DENSITY_GPML), 'g/mL');
        13 : ReportMessageBox_Alt('Density',
               PUnit^.Density.GetData(DENSITY_GPL), 'g/L');
        14 : ReportMessageBox_Alt('Density',
               PUnit^.Density.GetData(DENSITY_GPKL), 'g/kL');
        15 : ReportMessageBox_Alt('Density',
               PUnit^.Density.GetData(DENSITY_KGPL), 'kg/L');
        16 : ReportMessageBox_Alt('Temperature',
               PUnit^.Temperature.GetData(TEMPERATURE_KELVIN), 'Kelvin');
        17 : ReportMessageBox_Alt('Temperature',
              PUnit^.Temperature.GetData(TEMPERATURE_C), 'C');
        18 : ReportMessageBox_Alt('Temperature',
               PUnit^.Temperature.GetData(TEMPERATURE_F), 'F');
        19 : ReportMessageBox_Alt('Pressure',
               PUnit^.Pressure.GetData(PRESSURE_KPA), 'kPa');
        20 : ReportMessageBox_Alt('Pressure',
               PUnit^.Pressure.GetData(PRESSURE_PA), 'Pa');
        21 : ReportMessageBox_Alt('Pressure',
               PUnit^.Pressure.GetData(PRESSURE_ATM), 'atm');
        22 : ReportMessageBox_Alt('Pressure',
               PUnit^.Pressure.GetData(PRESSURE_TORR), 'torr');
      end; {case}

      {get the data from the input lines again so that new changes are
       discarded}
      PUnit^.GetRec;

    end; {else}
  end; {with DataRec}
end; {TProjectWindow.Calculate}

{----------------------------------------------------------------------------}

procedure TProjectWindow.ClearAllData;
begin
  if MessageBox('Are you sure that you wish to clear all data?', nil,
    mfConfirmation+mfYesNoCancel) = cmYes
  then
    PEquation^.ClearAllData;
end;

{----------------------------------------------------------------------------}

procedure TProjectWindow.DataEntry(Which: byte; IsReactant:boolean);
begin
  GetApplication^.ExecuteDialog(new(PTDataEntryDialog, Init(PEquation, Which,
    IsReactant)), nil);
end;

{----------------------------------------------------------------------------}

function TProjectWindow.GetPalette: PPalette;
const p : TPalette = CDialog;
begin
  GetPalette := @p;
end;

{----------------------------------------------------------------------------}

{The event handler for project window}

procedure TProjectWindow.HandleEvent(var Event: TEvent);
begin
  TWindow.HandleEvent(Event);
  case Event.What of
    evCommand:
      begin
        case Event.Command of
          cmClose     : Close;
          cmSetYield  : SetYield;
          cmCalculate : Calculate;
          cmClearAllData : ClearAllData;
          cmR1Button  : DataEntry(1, TRUE);
          cmR2Button  : DataEntry(2, TRUE);
          cmR3Button  : DataEntry(3, TRUE);
          cmR4Button  : DataEntry(4, TRUE);
          cmR5Button  : DataEntry(5, TRUE);
          cmR6Button  : DataEntry(6, TRUE);
          cmR7Button  : DataEntry(7, TRUE);
          cmR8Button  : DataEntry(8, TRUE);
          cmR9Button  : DataEntry(9, TRUE);

          cmP1Button  : DataEntry(1, FALSE);
          cmP2Button  : DataEntry(2, FALSE);
          cmP3Button  : DataEntry(3, FALSE);
          cmP4Button  : DataEntry(4, FALSE);
          cmP5Button  : DataEntry(5, FALSE);
          cmP6Button  : DataEntry(6, FALSE);
          cmP7Button  : DataEntry(7, FALSE);
          cmP8Button  : DataEntry(8, FALSE);
          cmP9Button  : DataEntry(9, FALSE);
        else
          Exit;
        end;
        ClearEvent(Event);
      end;
  end;
end;

{----------------------------------------------------------------------------}

function TProjectWindow.RealToStr(i : real) : string;
var
  EPos : byte;
  Ez : string;
  nf : real;
  nz : string;
  SF : shortInt;

begin
  if i = 0 then begin
    RealToStr := '0';
    exit;
  end;

  if boolean(CalculateDataRec.SF) then begin
    Str(i, nz);
    stripCh(' ', nz);

    {If SF unset, then return untrimmed result}
    SF := PEquation^.GetSF;
    if SF = UNSET then begin
      RealToStr := nz;
      exit;
    end;

    {Extract the scientific notation part}
    EPos := Pos('E', strupr(nz));
    Ez := extract(nz, EPos, length(nz)-EPos+1);

    {Remove trailing zeroes}
    repeat
      if (nz[length(nz)] = '0') then
        system.delete(nz, length(nz), 1)
      else
        break;
    until FALSE;

    {Remove decimal point}
    stripCh('.', nz);

    {Takes care of rounding 2.45 to 2.40, 2.50 to 2.60}
    if SF < length(nz) then
      if (nz[SF+1] = '5') and (SF+1 = length(nz)) and (atol(nz[SF]) mod 2 = 0)
      then
        nz[SF+1] := '0';

    nz := ltoa(round(atof('1.' + nz) * pow10(SF)));

    if SF > 1 then
      system.insert('.', nz, 3);

    if extract(nz, 1, 1) <> '1' then begin
      nz[1] := '1';
      Str(atof('1.0' + Ez) * 10, Ez);
      Ez := copy(Ez, length(Ez)-3, 4);
    end;

    {If Ez = E+00, kill it}
    If Ez = 'E+00' then
      Ez := '';

    RealToStr := nz + Ez;

  end else begin
  {if SF is not switched on}

    if (i >= 1e8) or (i <= -1e8) then
      Str(i, nz)
    else
      Str(i:20:20, nz);
    StripCh(' ', nz);
    RealToStr := nz;
  end;

end;

{----------------------------------------------------------------------------}

{Constructs a "set yield" dialog box and set PEquation^.Yield to the user
 input. Note that the user must enter a real number between 0 and 1.}

procedure TProjectWindow.SetYield;
var
  DataRec : record
    YieldInputLine : String[13]; {Inputline}
  end;
  nf : real;

begin
  repeat

    {Set up the input line with the old yield}

    Str(PEquation^.Yield:13:11, DataRec.YieldInputLine);
    StripCh(' ', DataRec.YieldInputLine);

    if GetApplication^.ExecuteDialog(PDialog(RezFile.Get('YieldEntryDialog')),
      @DataRec) = cmCancel
    then
      exit;

    nf := atof(DataRec.YieldInputLine);

    if (nf < 0) or (nf > 1) then
      MessageBox('Yield must be between 0 and 1', nil, mfError +
        mfOKButton)
    else begin
      PEquation^.Yield := nf;
      exit;
    end;
  until false;

end;

{----------------------------------------------------------------------------}

{This function adds the program's ability to display the "Are you sure to
 close the window?" message box when the user presses the "Close" button}

function TProjectWindow.Valid(Command: Word): Boolean;
begin
  Valid := TWindow.Valid(Command);
  if Command = cmClose then
    if MessageBox('Are you sure that you want to close the window?', nil,
      mfYesNoCancel) <> cmYes
    then
      Valid := FALSE;
end;

{----------------------------------------------------------------------------}

{Constructor to the TApp object. Opens the about box upon start of program}

constructor TApp.Init;
begin
  RezFile.Init(New(PTHaltStream, Init(GetExeBaseName + '.' + REZ_EXT, stOpen,
    REZ_BUFSIZE)));
  TApplication.Init;
  About;
end;

{----------------------------------------------------------------------------}

{Displays the "About" box}

procedure TApp.About;
begin
  ExecuteDialog(PDialog(RezFile.Get('AboutDialog')),nil);
end;

{----------------------------------------------------------------------------}

{This function adds the program's ability to display the "Are you sure to
 quit the Stoichiometry Calculator?" message box when the user selects to
 quit the program}

procedure TApp.EndModal(Command: Word);
begin
  if Command <> cmQuit then
    TApplication.EndModal(Command)
  else {IE. if command is cmQuit}
    if MessageBox('Are you sure that you want to quit the Stoichiometry '+
      'Calculator?', nil, mfYesNoCancel) = cmYes
    then
      TApplication.EndModal(Command)
end;

{----------------------------------------------------------------------------}

{The event handler for the application}

procedure TApp.HandleEvent(var Event: TEvent);
begin
  TApplication.HandleEvent(Event);
  case Event.What of
    evCommand:
      begin
        case Event.Command of
          cmAbout: about;
          cmCalc: InsertWindow(New(PCalculator, Init));
          cmNew: newWindow;
          cmCloseAll : Message(Desktop, evBroadcast, cmClose, nil);


        else
          Exit;
        end;
        ClearEvent(Event);
      end;
  end;
end;

{----------------------------------------------------------------------------}

procedure TApp.InitMenuBar;
begin
  MenuBar := PMenuBar(RezFile.Get('MenuBar'));
end;

{----------------------------------------------------------------------------}

procedure TApp.InitStatusLine;
begin
  StatusLine := PStatusLine(RezFile.Get('StatusLine'));
end;

{----------------------------------------------------------------------------}

{This procedure is called when the user chooses the "New" option}

procedure TApp.NewWindow;
var
  DataRec : record
    Equation : String[255]; {Inputline}
  end;
  PNewEquation : PTEquation;
  retry : boolean;

Begin
  {Start the input loop}
  PNewEquation := nil;
  DataRec.Equation := '';


  repeat

    {Executes the dialog box. If Cancel is pressed, then exit}
    if ExecuteDialog(PDialog(RezFile.Get('EquationEntryDialog')), @DataRec) =
      cmCancel
    then
      exit;

    if PNewEquation <> nil then
      dispose(PNewEquation, done);
    new(PNewEquation, Init);
    PNewEquation^.parse(DataRec.Equation);

    {If the equation cannot be analysed then}
    if PNewEquation^.isError then
      MessageBox('The Equation is invalid!', nil, mfError + mfOKButton)

    {If the equation is unbalanced then}
    else if not PNewEquation^.isBalanced then
      if MessageBox('The Equation is not balanced. Edit?', nil,
        mfWarning + mfYesButton + mfNoButton) = cmNo
      then
        break
      else

    else break;

  until FALSE;

  {Insert a project window into the desktop}
  InsertWindow(new(PTProjectWindow, Init(PNewEquation)));
end;

{----------------------------------------------------------------------------}

{Yes, the actual program takes only 3 lines}

begin
  VApp.init;
  VApp.run;
  VApp.done;
end.


{ Revision history:

  1.00b - Initial version

  1.00 - Significant digits
       - Stay on same items in 'Calculate' box
       - Standard temperature in Kelvin instead of C

  1.01 - Redesigned data entry box to accomodate Density and "Gas collected
         over water?" radio buttons.
       - Added ALT-F3 Close button.
       - Used resources.

  1.02 - Length of an equation increased to 255.
       - Up to 9 reactants and 9 products are now supported.
       - Inputting more than allowed reactants or products would not result
         in a system crash.
       - Example clarified.

}